//
//  PTRDebugPositionSimulator.h
//  PointrKit
//
//  Created by Ahmet Gülden on 31.05.2021.
//

#import <CoreLocation/CoreLocation.h>

#import <PointrKit/PTRPositioningTypes.h>

@class PTRLevel;

/// A simulator that simulates the devices location. It can provide location(s) to position manager.
@interface PTRDebugPositionSimulator : NSObject

/// Coordinates of the simulated location.
@property (nonatomic, assign) CLLocationCoordinate2D coordinate;

/// Level of the simulated location. Default value is nil.
@property (nonatomic, strong, nullable) PTRLevel *level;

/// Type of the simulated location. Default value is indoor.
@property (nonatomic, assign) PTRLocationType type;

/// State of the simulated location. Default value is active.
@property (nonatomic, assign) PTRLocationState state;

/// Accuracy of the simulated location. Default value is 5.0.
@property (nonatomic, assign) double accuracy;

/// Heading of the simulated location. Default value is 0.0.
@property (nonatomic, assign) double heading;

/// Heading accuracy of the simulated location. Default value is pi / 6.
@property (nonatomic, assign) double headingAccuracy;

/// Heading accuracy class of the simulated location. Default value is high.
@property (nonatomic, assign) PTRHeadingAccuracyClass headingAccurracyClass;

/// It is not possible to create a new instance from using init. A position simulator can be accessed
/// through PointrDebug object which is located at base Pointr class.
- (nonnull instancetype)init NS_UNAVAILABLE;

/// Resets all variables to their default values.
/// @param level Level of the simulated location.
- (void)resetVariablesForLevel:(nullable PTRLevel *)level;

/// Increases the level by one. If the level is already at the top of its building, this method moves
/// level to the lowest.
- (void)upOneLevel;

/// Decreases the level by one. If the level is already at the bottom of its building, this method moves
/// level to the highest.
- (void)downOneLevel;

/// Starts providing position to position manager periodically with interval of 0.1 seconds.
- (void)startSendingPositionsPeriodically;

/// Starts providing position to position manager periodically with given interval in seconds.
/// @param interval Interval of sending positions periodically.
- (void)startSendingPositionsPeriodicallyWithIntervals:(NSTimeInterval)interval;

/// Stops providing position to position manager periodically.
- (void)stopSendingPositionsPeriodically;

/// Provides a single location object to position manager.
- (void)sendLocationToPositionManager;

/// Updates the location for x and y deltas.
- (void)updatePositionWithDeltaX:(double)deltaX deltaY:(double)deltaY;

- (void)sendInvalidLocation;

@end
