//
//  PointrDebug.h
//  PointrKit
//
//  Created by Pointr on 21/05/2021.
//

#import <Foundation/Foundation.h>
#import <CoreLocation/CoreLocation.h>

#import <PointrKit/PTRAdvertiser.h>

NS_ASSUME_NONNULL_BEGIN

@class PTRGeofence;
@class PTRMappedBeacon;
@class PTRDebugPositionSimulator;
@class PTRPoi;

@protocol PTRPointrDebugDelegate <NSObject>

@optional

- (void)debugCoreLocationDidUpdateLocations:(NSArray<CLLocation *> *)locations;
- (void)debugCoreLocationDidRangeBeacon:(CLBeacon *)beacon withTimestamp:(NSDate *)timestamp;
- (void)debugCoreLocationDidEnterRegion:(CLRegion *)region;
- (void)debugCoreLocationDidExitRegion:(CLRegion *)region;
- (void)debugCoreLocationDidStartGpsLocationTracking:(CLLocationAccuracy)accuracy;
- (void)debugCoreLocationDidStartMonitoringGpsRegions:(NSSet<CLCircularRegion *> *)regions;
- (void)debugCoreLocationCurrentMonitoredGpsRegions:(NSSet<CLCircularRegion *> *)regions;

@end

@interface PointrDebug : PTRAdvertiser <id<PTRPointrDebugDelegate>>

/// A simulator that simulates the devices location. It provides location(s) to position manager.
@property (nonatomic, strong, readonly) PTRDebugPositionSimulator *positionSimulator;

/// The current GPS location accuracy.
@property (assign, nonatomic) CLLocationAccuracy currentGpsAccuracy;

- (nonnull instancetype)init NS_UNAVAILABLE;

/**
 * This method simulates GPS updates outside of the sites for 5 seconds.
 * @param duration The duration of the sumulation in seconds.
 */
- (void)startSimulatingOutsideLocation:(double)duration;

/**
 * This method simulates GPS updates on the given coordinates.
 * @param coordinates The coordinates to simulate.
 * @param duration The duration of the sumulation in seconds.
 */
- (void)startSimulatingCoordinates:(CLLocationCoordinate2D)coordinates duration:(double)duration;

/**
 * This method simulates a beacon for a certain duration
 * @param beacon The beacon to simulate.
 * @param duration The duration of the sumulation in seconds.
 */
- (void)startSimulatingBeacon:(PTRMappedBeacon *)beacon duration:(double)duration;

/**
 * This method simulates an enter event for a given geofence.
 * @param geofence The geofence for which the enter event is simulated.
 */
- (void)simulateEnterEventForGeofence:(PTRGeofence *)geofence;

/**
 * This method simulates an exit event for a given geofence.
 * @param geofence The geofence for which the exit event is simulated.
 */
- (void)simulateExitEventForGeofence:(PTRGeofence *)geofence;

/// This method simulates an enter event for a given indoor geofence.
/// @param indoorGeofence The geofence for which the enter event is simulated.
- (void)simulateEnterEventForIndoorGeofence:(PTRPoi *)indoorGeofence;

/// This method simulates an exit event for a given indoor geofence.
/// @param indoorGeofence The geofence for which the exit event is simulated.
- (void)simulateExitEventForIndoorGeofence:(PTRPoi *)indoorGeofence;

/// Method to change gps locatioj tracking accuracy
/// @param accuracy The accuracy to be set
- (void)setAccuracy:(CLLocationAccuracy)accuracy;

@end

NS_ASSUME_NONNULL_END
